
""" Hostory: see README """

__version__ = (0, 3, 0, 'beta')

# for setuptools
# - Only use . as a separator
# - No spaces: (0, 1, 0, 'beta') -> "0.1.0beta"
# http://peak.telecommunity.com/DevCenter/setuptools#specifying-your-project-s-version
VERSION_STRING = "%s.%s.%s%s" % __version__
